#!/usr/bin/python3

import sys
import re

# This probably isn't a good way to do this, but pandoc
# outputs fairly consistently formatted style information,
# so we should be able to parse it consistently like this
code_general_re = re.compile(r'code\s*\{(?P<style>.*?)\}',
        re.MULTILINE | re.DOTALL)
# Ignoring "code" prefix on the premise that the classes
# should only be used in the correct place (I think)
code_span_re = re.compile(r'^code > span\.(?P<spanclass>\S+) \{(?P<style>.*?)\}',
        re.MULTILINE | re.DOTALL)
other_style_re = re.compile(r'^(?P<stylelist>[^\n]*?) \{(?P<style>.*?)\}\s*$',
        re.MULTILINE | re.DOTALL)

# Copy code assumes a fragment without head & tail, but we need
# pandoc in standalone mode to get the style information, so these
# REs get rid of the header and footer to make it a fragment again.
pre_body_re = re.compile(r'^.*?<body>', re.DOTALL | re.IGNORECASE)
post_body_re = re.compile(r'</body>.*$', re.DOTALL | re.IGNORECASE)

def find_styles(lines):
    styles = {}
    general = code_general_re.finditer(lines)
    codespan = code_span_re.finditer(lines)
    other = other_style_re.finditer(lines)

    for m in general:
        styles["code"] = m.group("style").replace("\n", " ")
    for m in codespan:
        styles["span."+m.group("spanclass")] = m.group("style").replace("\n", " ")
    for m in other:
        groups = m.group("stylelist").split(", ")
        for group in groups:
            styles[group] = m.group("style").replace("\n", " ")
    return styles

def map_styles(lines, styles):
    modified = lines
    for classinfo, style in styles.items():
        parts = classinfo.split(".")
        if len(parts) == 1:
            matcher = re.compile(r'(<%s)( )' % classinfo)
        else:
            matcher = re.compile(r'(<%s)( class="%s[ "])' % tuple(parts))
        modified = matcher.sub(r'\1 style="%s"\2' % style.strip(), modified)
    return modified

def remove_header_and_footer(content):
    modified = pre_body_re.sub("", content)
    modified = post_body_re.sub("", modified)
    return modified

def convert_styles(in_lines):
    styles = find_styles(in_lines)
    styles_mapped = map_styles(in_lines, styles)
    just_body = remove_header_and_footer(styles_mapped)
    return just_body

def main():
    in_lines = sys.stdin.read()
    print(convert_styles(in_lines))

if __name__ == "__main__":
    main()
