" Markdown Copy:
"  Author:  A. S. Budden <abudden _at_ gmail _dot_ com>
" Copyright: Copyright (C) 2017 A. S. Budden
"            Permission is hereby granted to use and distribute this code,
"            with or without modifications, provided that this copyright
"            notice is copied with it. Like anything else that's free,
"            the MarkdownCopy plugin is provided *as is* and comes with no
"            warranty of any kind, either expressed or implied. By using
"            this plugin, you agree that in no event will the copyright
"            holder be liable for any damages resulting from the use
"            of this software.
" ---------------------------------------------------------------------

if v:version < 800
	finish
endif

" To get this working on Linux, need to use something other than
" pywin32
if ! has("win32")
	finish
endif

try
	if &cp || (exists('g:loaded_MarkdownCopy') && (g:plugin_development_mode != 1))
		throw "Already loaded"
	endif
catch
	finish
endtry
let g:loaded_MarkdownCopy = 1

let g:MarkdownCopyPluginPathFixed = 0
let g:MarkdownCopyPluginsImported = 0
let s:plugin_paths = split(globpath(&rtp, 'plugin/MarkdownCopy/HtmlClipboard.py'), '\n')
if len(s:plugin_paths) == 1
	let g:MarkdownCopyPluginPath = fnamemodify(s:plugin_paths[0], ':p:h')
elseif len(s:plugin_paths) == 0
	echoerr "Cannot find HtmlClipboard.py"
else
	echoerr "Multiple plugin installs found: something has gone wrong!"
endif

if ! exists('g:MarkdownCopyConverter')
	let g:MarkdownCopyConverter = "pandoc"
endif
if ! exists('g:MarkdownCopyConvertToHtml')
	let g:MarkdownCopyConvertToHtml = g:MarkdownCopyConverter
				\ . " HIGHLIGHT_PLACEHOLDER"
endif
if ! exists('g:MarkdownCopyHighlightStyle')
	let g:MarkdownCopyHighlightStyle = "haddock"
endif
if ! exists('g:MarkdownCopyConvertFromHtml')
	let g:MarkdownCopyConvertFromHtml = g:MarkdownCopyConverter 
				\ . " -f html -t markdown"
endif
if ! exists('g:MarkdownCopyDebugLevel')
	let g:MarkdownCopyDebugLevel = 0
endif
function! s:FixPythonPath()
	if g:MarkdownCopyPluginPathFixed == 1
		return
	endif

	pyx import sys
	pyx sys.path.append(vim.vars["MarkdownCopyPluginPath"].decode("utf8"))
	pyx import HtmlClipboard

	let g:MarkdownCopyPluginPathFixed = 1
endfunction

function! s:CheckOptionalDependencies()
	if g:MarkdownCopyPluginsImported == 1
		return
	endif

pyx << EOF
try:
	from inlinestyler.utils import inline_css
	from ClassFilter import remove_header_and_footer
	convert_styles = lambda x: remove_header_and_footer(inline_css(x))
	vim.vars["temp__markdowncopy_import_success"] = 1
except ImportError:
	from ClassFilter import convert_styles
	vim.vars["temp__markdowncopy_import_success"] = 0
EOF
	let g:MarkdownCopyPluginsImported = 1

	if ! exists("g:MarkdownCopyInlineStyles")
		let g:MarkdownCopyInlineStyles = g:temp__markdowncopy_import_success
	endif
	unlet g:temp__markdowncopy_import_success
endfunction

if ! exists('g:MarkdownCopyInlineStyles')
	if g:MarkdownCopyConverter == "pandoc"
		" Should be safe regardless
		let g:MarkdownCopyInlineStyles = 1
	else
		call s:FixPythonPath()
		call s:CheckOptionalDependencies()
	endif
endif

function! s:CopyToClipboardAsHtml() range
	call s:FixPythonPath()
	call s:CheckOptionalDependencies()
	pyx startline = vim.eval("a:firstline")
	pyx endline = vim.eval("a:lastline")

	let g:temp__markdowncopy_command =
				\ substitute(g:MarkdownCopyConvertToHtml,
				\     "HIGHLIGHT_PLACEHOLDER",
				\     "--highlight-style " . g:MarkdownCopyHighlightStyle,
				\     "")
	if g:MarkdownCopyInlineStyles != 0
		let g:temp__markdowncopy_command .= " -s"
	endif
	if g:MarkdownCopyDebugLevel >= 1
		echomsg g:temp__markdowncopy_command
	endif
	pyx mdc_filecontents = vim.eval("system(g:temp__markdowncopy_command, getline({}, {}))".format(startline, endline))
	unlet g:temp__markdowncopy_command

	if g:MarkdownCopyInlineStyles != 0
		pyx mdc_filecontents = convert_styles(mdc_filecontents)
	endif
	pyx HtmlClipboard.PutHtml(mdc_filecontents)
	if g:MarkdownCopyDebugLevel >= 2
		pyx print(mdc_filecontents)
	endif
endfunction

function! s:PasteFromClipboardAsMarkdown()
	call s:FixPythonPath()
	pyx html_from_clipboard = HtmlClipboard.GetHtml()
	pyx vim.vars["MarkdownCopyHtmlFromClipboard"] = html_from_clipboard if html_from_clipboard else ""

	if len(g:MarkdownCopyHtmlFromClipboard) > 0
		let md = system(g:MarkdownCopyConvertFromHtml, g:MarkdownCopyHtmlFromClipboard)
		call append(".", split(md, '\v\n'))
	endif
endfunction

command! PasteFromClipboardAsMarkdown call s:PasteFromClipboardAsMarkdown()
command! -range=% CopyToClipboardAsHtml <line1>,<line2>call s:CopyToClipboardAsHtml()
